@@ -5,7 +5,7 @@ from operator import itemgetter
 
 from pyelasticsearch import ElasticSearch
 
-from elasticutils._version import __version__
+from elasticutils._version import __version__  # noqa
 
 
 log = logging.getLogger('elasticutils')
@@ -366,7 +366,7 @@ class PythonMixin(object):
                 # the results.
                 try:
                     return datetime.strptime(obj, '%Y-%m-%dT%H:%M:%S')
-                except ValueError:
+                except (TypeError, ValueError):
                     pass
 
         elif isinstance(obj, dict):
@@ -539,7 +539,7 @@ class S(PythonMixin):
         [(1, 'fred', 40), (2, 'brian', 30), (3, 'james', 45)]
         >>> list(S().values_list('id', 'name'))
         [(1, 'fred'), (2, 'brian'), (3, 'james')]
-        >>> list(S().values_list('name', 'id')
+        >>> list(S().values_list('name', 'id'))
         [('fred', 1), ('brian', 2), ('james', 3)]
 
         .. Note::
@@ -567,7 +567,7 @@ class S(PythonMixin):
 
         >>> list(S().values_dict())
         [{'id': 1, 'name': 'fred', 'age': 40}, ...]
-        >>> list(S().values_dict('id', 'name')
+        >>> list(S().values_dict('id', 'name'))
         [{'id': 1, 'name': 'fred'}, ...]
 
         """
@@ -629,7 +629,7 @@ class S(PythonMixin):
         Examples:
 
         >>> s = S().query(foo='bar')
-        >>> s = S().query(Q(foo=='bar'))
+        >>> s = S().query(Q(foo='bar'))
         >>> s = S().query(foo='bar', bat__text='baz')
         >>> s = S().query(foo='bar', should=True)
         >>> s = S().query(foo='bar', should=True).query(baz='bat', must=True)
@@ -723,6 +723,26 @@ class S(PythonMixin):
         """
         return self._clone(next_step=('filter', list(filters) + kw.items()))
 
+    def filter_raw(self, filter_):
+        """
+        Return a new S instance with a filter_raw.
+
+        :arg filter_: Python dict specifying the complete filter to send
+            to Elasticsearch
+
+        Example::
+
+            S().filter_raw({'term': {'title': 'example'}})
+
+
+        .. Note::
+
+           If there's a filter_raw in your S, then that's your
+           filter. All ``.filter()`` and anything else that affects the
+           filter clause is ignored.
+        """
+        return self._clone(next_step=('filter_raw', filter_))
+
     def boost(self, **kw):
         """
         Return a new S instance with field boosts.
@@ -907,6 +927,7 @@ class S(PythonMixin):
         Elasticsearch, and return it as a dict.
         """
         filters = []
+        filters_raw = None
         queries = []
         query_raw = None
         sort = []
@@ -950,6 +971,8 @@ class S(PythonMixin):
                 demote = value
             elif action == 'filter':
                 filters.extend(self._process_filters(value))
+            elif action == 'filter_raw':
+                filters_raw = value
             elif action == 'facet':
                 # value here is a (args, kwargs) tuple
                 facets.update(_process_facets(*value))
@@ -970,10 +993,15 @@ class S(PythonMixin):
                 raise NotImplementedError(action)
 
         qs = {}
-        if len(filters) > 1:
-            qs['filter'] = {'and': filters}
-        elif filters:
-            qs['filter'] = filters[0]
+
+        # If there's a filters_raw, we use that.
+        if filters_raw:
+            qs['filter'] = filters_raw
+        else:
+            if len(filters) > 1:
+                qs['filter'] = {'and': filters}
+            elif filters:
+                qs['filter'] = filters[0]
 
         # If there's a query_raw, we use that. Otherwise we use
         # whatever we got from query and demote.
@@ -1100,6 +1128,10 @@ class S(PythonMixin):
                 elif field_action in RANGE_ACTIONS:
                     rv.append({'range': {key: {field_action: val}}})
 
+                elif field_action == 'range':
+                    lower, upper = val
+                    rv.append({'range': {key: {'gte': lower, 'lte': upper}}})
+
                 else:
                     raise InvalidFieldActionError(
                         '%s is not a valid field action' % field_action)
@@ -1134,9 +1166,7 @@ class S(PythonMixin):
             # Note: query_string queries are not boosted with
             # .boost()---they're boosted in the query text itself.
             return {
-                'query_string':
-                    {'default_field': field_name,
-                     'query': val}
+                'query_string': {'default_field': field_name, 'query': val}
             }
 
         elif field_action in RANGE_ACTIONS:
@@ -1145,7 +1175,18 @@ class S(PythonMixin):
             return {
                 'range': {field_name: _boosted_value(
                         field_action, field_action, key, val, boost)}
+           }
+
+        elif field_action == 'range':
+            lower, upper = val
+            value = {
+                'gte': lower,
+                'lte': upper,
             }
+            if boost:
+                value['boost'] = boost
+
+            return {'range': {field_name: value}}
 
         raise InvalidFieldActionError(
             '%s is not a valid field action' % field_action)
@@ -1357,7 +1398,6 @@ class S(PythonMixin):
         count = self.count()
         return self[:count].execute()
 
-
     def execute(self):
         """
         Executes search and returns a `SearchResults` object.
@@ -1905,7 +1945,7 @@ class Indexable(object):
         raise NotImplemented
 
     @classmethod
-    def index(cls, document, id_=None, force_insert=False, es=None,
+    def index(cls, document, id_=None, overwrite_existing=False, es=None,
               index=None):
         """Adds or updates a document to the index
 
@@ -1924,7 +1964,7 @@ class Indexable(object):
                will make up an id for your document and it'll look
                like a character name from a Lovecraft novel.
 
-        :arg force_insert: TODO
+        :arg overwrite_existing: TODO
 
         :arg es: The `ElasticSearch` to use. If you don't specify an
             `ElasticSearch`, it'll use `cls.get_es()`.
@@ -1950,7 +1990,7 @@ class Indexable(object):
             cls.get_mapping_type_name(),
             document,
             id=id_,
-            force_insert=force_insert)
+            overwrite_existing=overwrite_existing)
 
     @classmethod
     def bulk_index(cls, documents, id_field='id', es=None, index=None):
