@@ -1,10 +1,10 @@
 from nose.tools import eq_
 
 from elasticutils import MLT
-from elasticutils.tests import ElasticTestCase
+from elasticutils.tests import ESTestCase
 
 
-class MoreLikeThisTest(ElasticTestCase):
+class MoreLikeThisTest(ESTestCase):
     @classmethod
     def setup_class(cls):
         super(MoreLikeThisTest, cls).setup_class()
@@ -42,7 +42,7 @@ class MoreLikeThisTest(ElasticTestCase):
         doc_type = self.get_s().get_doctypes()[0]
         es = self.get_s().get_es()
 
-        mlt = MLT(1, fields=['foo'], index=index, doctype=doc_type,
+        mlt = MLT(1, mlt_fields=['foo'], index=index, doctype=doc_type,
                   es=es, min_term_freq=1, min_doc_freq=1)
         eq_(len(mlt), 4)
 
@@ -59,6 +59,15 @@ class MoreLikeThisTest(ElasticTestCase):
                   min_term_freq=1, min_doc_freq=1)
         eq_(len(mlt), 5)
 
+    def test_mlt_deprecated_fields(self):
+        with self.assertRaises(DeprecationWarning):
+            MLT(1, self.get_s(), fields=['tag', 'foo'])
+
+    def test_mlt_iter(self):
+        mlt = MLT(1, self.get_s(), ['tag', 'foo'],
+                  min_term_freq=1, min_doc_freq=1)
+        eq_(len(list(mlt)), 5)
+
     def test_mlt_on_foo_with_filter(self):
         """Verify MLT with the foo field while filtering on tag."""
         # We need to pass min_term_freq and min_doc_freq, because the terms
