@@ -1,3 +1,27 @@
+from itertools import islice
+
+
+def chunked(iterable, n):
+    """Returns chunks of n length of iterable
+
+    If len(iterable) % n != 0, then the last chunk will have length
+    less than n.
+
+    Example:
+
+    >>> chunked([1, 2, 3, 4, 5], 2)
+    [(1, 2), (3, 4), (5,)]
+
+    """
+    iterable = iter(iterable)
+    while 1:
+        t = tuple(islice(iterable, n))
+        if t:
+            yield t
+        else:
+            return
+
+
 def format_explanation(explanation, indent='  ', indent_level=0):
     """Return explanation in an easier to read format
 
